﻿import flash.geom.Rectangle;
import flash.geom.Point;
import flash.geom.Matrix;

class PhotoController implements IPhotoController{
	
	//created here (though,only if the owner says to in p_clipsToLoad);
	private var displayarea_mc:MovieClip;//IDisplayArea;
	private var chapters_mc:MovieClip;//IChapters;
	private var thumbnails_mc:MovieClip;//IThumbnails;
	
	
	private var scope:MovieClip;
	private var imageData:Array;//of ImageInfo objs
	private var chapterData:Array;//of chapterInfo objs
	private var clipsToLoad:Array;//of ClipInfo objs 
	
	
	private var currentChapter:ChapterInfo;
	private var currentImage:ImageInfo;
		
	private var mcl:MovieClipLoader;
	
	
	
	function PhotoController(	 p_scope:MovieClip,
								 p_clipsToLoad:Array,
								 p_imageData:Array,
								 p_chapterData:Array){
									
	
		scope = p_scope;
		clipsToLoad = p_clipsToLoad;
		imageData = p_imageData;
		chapterData = p_chapterData;
		
		currentChapter = chapterData[0];
		
		//Make clip holders/load clips:
		var total = clipsToLoad.length;
		
		mcl = new MovieClipLoader();		
		var clipsLoadedSoFar = 0;
		
		var primaryListener = new Object();
		primaryListener.controller = this;
		
		//when fully loaded and ready to go, give clip a reference to us
		primaryListener.onLoadInit = function (target){
			target.bars.removeMovieClip();
			target.setController(this.controller);
			clipsLoadedSoFar++;
			if(clipsLoadedSoFar == total){
				this.controller.allClipsLoaded();
			}
		}
		//=========forwarded to the bars created
		primaryListener.onLoadStart = function(target){
			var total=barsCreated.length;
			for(var i = 0; i<total; i++){
				barsCreated[i].onLoadStart(target);
			}
		}
		//=========forwarded to the bars created
		primaryListener.onLoadProgress = function(target,bytesLoaded,bytesTotal){
			var total=barsCreated.length;
			for(var i = 0; i<total; i++){
				barsCreated[i].onLoadProgress(target,bytesLoaded,bytesTotal);
			}
		}
		//=========forwarded to the bars created
		primaryListener.onLoadComplete = function(target){
			var total=barsCreated.length;
			for(var i = 0; i<total; i++){
				barsCreated[i].onLoadComplete(target);
			}
		}
		
		mcl.addListener(primaryListener);
			
			
		//holder for ALL progress bars (above clips)
		scope.createEmptyMovieClip("bars",total);
		
		var barsCreated:Array = new Array();
		var listenerArray = new Array();
		for(var i = 0; i<total; i++){
			var thisClip:ClipInfo = clipsToLoad[i];

			//make an instance
			var clipName = thisClip.type.toLowerCase()+"_mc";
			
			var clip:MovieClip = scope.createEmptyMovieClip(clipName,i);
			clip.transform.matrix = new Matrix(1,0,0,1,thisClip.rect.x,thisClip.rect.y)
			
			//save a reference in this class's vars
			this[clipName] = clip;
			
			//========================progress bar 
			barsCreated.push(  scope.bars.attachMovie("ProgressBar","bar"+i, i,
								{rect:thisClip.rect, target:clip}));
								
			
			//load the swf:	
			mcl.loadClip(thisClip.url, clip);
			
		}
		
	}
	private function allClipsLoaded(){
		//default to first chapter
		currentChapter = chapterData[0]
		
		//resize and mask them all
		var total = clipsToLoad.length;
		for(var i = 0; i<total; i++){
			var thisClipInfo:ClipInfo = clipsToLoad[i];
			var thisClip:MovieClip =this[thisClipInfo.type.toLowerCase()+"_mc"];
			
			//tell the clip to resize
			thisClip.sizeToRect(thisClipInfo.rect);
		
			var  thisMask:MovieClip = scope.createEmptyMovieClip("mask"+i, scope.getNextHighestDepth());
			
			var shiftedRect:Rectangle = thisClipInfo.rect.clone();
			shiftedRect.offset(thisClip._x, thisClip._y);
			fillClipWithRect(thisMask, shiftedRect,  0xFF0000);
			thisClip.setMask(thisMask);
		}
		
		
		
		var STEP = 2;
		var FREQUENCY = 10;
		//scope.bars.removeMovieClip();
		var interval = setInterval(function(ref:MovieClip){
								ref._alpha -= STEP;
								if(ref._alpha < 0 ){
									ref.removeMovieClip();
									clearInterval(interval);
								}}, FREQUENCY, scope.bars);
									
	}
	
	//UTIL:
	private function fillClipWithRect(clip, rect, color){
		clip.beginFill(color);
		clip.lineTo(rect.left, rect.top);
		clip.lineTo(rect.left, rect.bottom);
		clip.lineTo(rect.right, rect.bottom);
		clip.lineTo(rect.right, rect.top);
		clip.lineTo(rect.left, rect.top);
		clip.endFill();
	}

	//REQUIRED BY INTERFACE:
	
	// ====================
	//called by thumbnails
	public function clickThumb(imageInfo:ImageInfo){
		currentImage = imageInfo;
		displayarea_mc.showImage(imageInfo);
		//don't need to tell thumbnails--they're the one who called this
	}
		
	// ====================
	//called by chapters:
	public function getChapterData():Array{
		return chapterData;
	}
	public function pickChapter(infoOrIndex){

		if ( isNaN(infoOrIndex) ){//then it must be a ChapterInfo obj
			currentChapter = infoOrIndex;
		}else{
			currentChapter = chapterData[infoOrIndex];
		}
		
		//clear current image
		currentImage = undefined;
		displayarea_mc.showImage(undefined);
		
		
		//tell thumbnails to update:
		thumbnails_mc.displayThumbs();
	}
	
	// ====================
	//called by all types
	public function getImageData():Array{
		var total = imageData.length;
		var retArray = new Array();
		for( var i = 0; i<total; i++){
			if( imageData[i].chapterID == currentChapter.ID ){
				retArray.push(imageData[i]);
			}
		}
		return retArray;
	}

	public function goNextImage(){
		//find current image and get the next one
		var foundIndex = 0;
		var total = imageData.length;
		for (var i = 0; i<total; i++){
			if (imageData[i] == currentImage){
				foundIndex = i+1;
				break;
			}
		}
		if (foundIndex >= total) foundIndex = 0;
		
		//if they just entered a chapter, the current image will be undefined
		//in that case grab the first image in the current chapter
		if (currentImage == undefined) {
			for (var i = 0; i<total; i++){
				if (imageData[i].chapterID == currentChapter.ID){
					foundIndex = i;
					break;
				}
			}
		}
		
		//save new image:
		currentImage = imageData[foundIndex];
		
		
		//added
		if(currentChapter.ID == undefined){
			displayarea_mc.showImage(currentImage);
			thumbnails_mc.showImage(currentImage);
			return;
		}
		
		
		//see if the chapter changed (and broadcast this info first)
		var chapterIDBeforeChange = currentChapter.ID;
		//get the 
		var newChapterID = currentImage.chapterID;
		var total = chapterData.length;
		for(var i = 0; i<total; i++){
			if (chapterData[i].ID == newChapterID){
				currentChapter = chapterData[i];
			}
		}
		if( currentChapter.ID != chapterIDBeforeChange){
			//tell chapters and thumbnails
			chapters_mc.updateChapter(currentChapter);
			thumbnails_mc.displayThumbs();
		}
		
		//tell everyone about the next image:
		displayarea_mc.showImage(currentImage);
		thumbnails_mc.showImage(currentImage);
		
		
		
	}
	
	//only for main_tester_app
	public function clear(){
		displayarea_mc.removeMovieClip();
		chapters_mc.removeMovieClip();
		thumbnails_mc.removeMovieClip();
	}
	
}

